//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2019 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package auth

import (
	"reflect"
	"testing"
	"time"
)

var (
	testSecret = "secrettest"
)

//
// Testing
//

func TestCreateSessionManager(t *testing.T) {
	cfg := SessionsConfig{}
	if _, err := NewSessionManager(cfg); err != nil {
		t.Fatalf("unexpected error: %v", err)
	}
	// TODO: would be nice to compare two session managers that were created
	//       with random keys and make sure they always produce new/different
	//       keys - however this does not work because the keys are unexported
	//       values and reflect.DeepEqual can't be used because some values of
	//       the securecockie struct are always different

	cfg.Secret = testSecret
	if _, err := NewSessionManager(cfg); err != nil {
		t.Fatalf("unexpected error: %v", err)
	}
	// TODO: same problem as above but now we would like to test if using the
	//       same secret twice actually leads to session stores that have the
	//       same key
}

func TestSessionExpiry(t *testing.T) {
	cfg := SessionsConfig{}
	cfg.Secret = testSecret
	cfg.MaxAge = time.Second
	sm, err := NewSessionManager(cfg)
	if err != nil {
		t.Fatalf("unexpected error: %v", err)
	}

	s := sm.NewSession()
	if s == nil {
		t.Fatalf("session manager returned nil instead of new session")
	}

	if s.Expired() {
		t.Fatalf("session has already expired")
	}

	time.Sleep(time.Second + 100*time.Millisecond)

	if !s.Expired() {
		t.Fatalf("session hasn't expired in time")
	}
}

func TestSessionSealUnseal(t *testing.T) {
	cfg := SessionsConfig{}
	cfg.Secret = testSecret
	sm, err := NewSessionManager(cfg)
	if err != nil {
		t.Fatalf("unexpected error: %v", err)
	}

	in := sm.NewSession()
	if in == nil {
		t.Fatalf("session manager returned nil instead of new session")
	}

	c, err := in.Seal()
	if err != nil {
		t.Fatalf("unexpected error: %v", err)
	}

	out, err := sm.UnsealSession(c)
	if err != nil {
		t.Fatalf("unexpected error: %v", err)
	}

	if !in.Expires.Equal(out.Expires) {
		t.Fatalf("session are not equal, expected expiry: %v, got %+v", in.Expires, out.Expires)
	}
	if in.Username != out.Username {
		t.Fatalf("session are not equal, expected username: %q, got %q", in.Username, out.Username)
	}
	if in.AllGroups != out.AllGroups {
		t.Fatalf("session are not equal, expected all-groups: %v, got %v", in.AllGroups, out.AllGroups)
	}
	if !reflect.DeepEqual(in.Groups, out.Groups) {
		t.Fatalf("session are not equal, expected groups: %v, got %v", in.Groups, out.Groups)
	}
}
